float4x4 view_proj_matrix;
float4x4 view_matrix;
float3 dlightPos;
float specularPower;

struct VS_OUTPUT 
{
   float4 Pos : POSITION;   
   float2 Tex : TEXCOORD0;
   float3 Norm : TEXCOORD2;
   float4 diffuse : COLOR0;
};

VS_OUTPUT vs_main( float4 inPos: POSITION, float2 inTex: TEXCOORD, float4 inNorm : NORMAL )
{
   VS_OUTPUT Out;

   Out.Pos = mul( view_proj_matrix, inPos );
   float3 view = mul( view_matrix, inPos );
   float3 viewNormal = mul((float3x3)view_matrix, (float3)inNorm);
   
   // get a specular vector.
   float3 vReflect = normalize((2 * dot(viewNormal, dlightPos) * viewNormal) - dlightPos);
   float4 specular =  pow(max(0, dot(vReflect, -normalize(view))), specularPower);
   
   Out.Norm = viewNormal;
   Out.Tex = inTex;

// store Craptacular Lighting Information

   Out.diffuse = specular;
   

   return Out;
}

sampler2D baseMap;
float4 ambient;

float4 ps_main( float4 inDiffuse: COLOR0, float2 inTex: TEXCOORD0, float3 inEnv : TEXCOORD1,
    float3 inNormal : TEXCOORD2,
    float4 inReflector : TEXCOORD3) : COLOR0
{

// FIXME: normalize cube map for reflector.

   float4 base = tex2D(baseMap, inTex);
   float4 diffuse = dot(dlightPos, inNormal);
   diffuse.w = 1;
      
   return base * diffuse + inDiffuse;
}
