float4x4 view_proj_matrix;
float4x4 view_matrix;
float4x4 normal_matrix;
float3 dlightPos;


struct VS_OUTPUT
{
   float4 Pos:     POSITION;
   float3 Norm   : TEXCOORD0;
   float3 View   : TEXCOORD1;
   float2 Tex : TEXCOORD2;
};

VS_OUTPUT vs_main( float4 inPos: POSITION, float4 inNorm : NORMAL, float2 inTex : TEXCOORD0, float2 
inTex2 : TEXCOORD1)
{
   VS_OUTPUT Out;

   Out.Pos = mul(view_proj_matrix, inPos);   
   Out.Tex = inTex;
   Out.Norm = mul(normal_matrix, inNorm);
   Out.View = mul(view_matrix, inPos);

   return Out;
}

// ----------------- BEGIN PIXEL SHADER CONSTANTS -------------------

sampler baseMap : register(s0);
float4 dlightColor;
float4 ambient;
float specularPower;

float4 ps_main( 
             float3 Normal : TEXCOORD0,
             float3 View   : TEXCOORD1,
             float2 Tex    : TEXCOORD2) : COLOR
{
	float4 FinalColor;

	Normal = normalize(Normal);
	float3 diffuse = dlightColor.xyz * dot(Normal, dlightPos);
	float3 vReflect = normalize((2 * dot(Normal, dlightPos) * Normal) - dlightPos);
	float3 specular =  dlightColor * pow(max(0, dot(vReflect, -normalize(View))), specularPower) * 0.5f;


	diffuse = saturate(diffuse + ambient.xyz);
	float4 albedo = tex2D(baseMap, Tex);
	FinalColor.xyz = specular + (diffuse * albedo);
	FinalColor.w = albedo.w;

	return FinalColor;
}

