float4x4 view_proj_matrix;
float4x4 view_matrix;
float3 dlightPos;
float specularPower;

struct VS_OUTPUT 
{
   float4 Pos : POSITION;   
   float2 Tex : TEXCOORD0;
   float3 EnvMap : TEXCOORD1;
   float3 Norm : TEXCOORD2;
};

VS_OUTPUT vs_main( float4 inPos: POSITION, float2 inTex: TEXCOORD, float4 inNorm : NORMAL )
{

   VS_OUTPUT Out;

   Out.Pos = mul( view_proj_matrix, inPos );
   float3 view = mul( view_matrix, inPos );
   float3 viewNormal = mul((float3x3)view_matrix, (float3)inNorm);

   float3 vReflecter = reflect(view, viewNormal);         // for envmap.
   
   // get a specular vector.
   
   float3 vReflect = normalize((2 * dot(viewNormal, dlightPos) * viewNormal) - dlightPos);
   
   Out.EnvMap = vReflecter;
   Out.Norm = viewNormal;
   Out.Tex = inTex;
   

   return Out;
}

sampler2D baseMap;
samplerCUBE envMap;
float4 ambient;

float4 ps_main( float2 inTex: TEXCOORD0, float3 inEnv : TEXCOORD1,
    float3 inNormal : TEXCOORD2,
    float4 inReflector : TEXCOORD3) : COLOR0
{

   float4 cubeVal = texCUBE(envMap, inEnv) * 0.5f;
   float4 base = tex2D(baseMap, inTex);
   float4 diffuse = dot(dlightPos, inNormal) + ambient;
      
   return (cubeVal + base) * diffuse;
}
